//
//  ViewController.m
//  Sit-up
//
//  Created by 敦史 掛川 on 12/03/16.
//  Copyright (c) 2012年 __MyCompanyName__. All rights reserved.
//

#import "ViewController.h"

// 仰角の判定時のマージン
static const int kPitchDegreeMargin = 15;

@implementation ViewController

@synthesize countLabel;
@synthesize button;
@synthesize arrowImage;

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
	
    // 初期化
    isCounting = NO;
    isRising = NO;
    [self changeViewState];
    
    // CMMotionManagerを作成
    motionManager = [[CMMotionManager alloc] init];
    motionManager.deviceMotionUpdateInterval = 0.1;
}

- (void)viewDidUnload
{
    [self setCountLabel:nil];
    [self setButton:nil];
    [self setArrowImage:nil];
    motionManager = nil;
    
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Private methods
 
- (void)changeViewState
{
    if (isCounting)
    {
        [button setTitle:@"ストップ" forState:UIControlStateNormal];
        arrowImage.hidden = NO;
    }
    else
    {
        [button setTitle:@"スタート" forState:UIControlStateNormal];
        arrowImage.hidden = YES;
        self.countLabel.text = nil;
    }
}

- (void)startCounting
{
    // モーションデータ更新時のハンドラを作成
    void (^handler)(CMDeviceMotion *, NSError *) = ^(CMDeviceMotion *motion, NSError *error)
    {
        // ピッチ角で腹筋動作の判定
        [self validatePitch:motion.attitude.pitch];
    };
    
    // モーションデータの測定を開始
    NSOperationQueue *queue = [NSOperationQueue currentQueue];
    [motionManager startDeviceMotionUpdatesToQueue:queue withHandler:handler];
    
    isCounting = YES;
    count = 0;
    isRising = NO;
    
    [self updateViewAnimated:NO];
    [self changeViewState];
}

- (void)stopCounting
{
    // モーションデータの測定を終了
    [motionManager stopDeviceMotionUpdates];
    
    isCounting = NO;
    [self changeViewState];
}

- (void)updateViewAnimated:(BOOL)animated
{
    // カウント回数ラベルの更新
    countLabel.text = [NSString stringWithFormat:@"%d回", count];
    
    // 矢印イメージの更新
    if (animated)
    {
        [UIView animateWithDuration:0.3 animations:^{
            [self rotateArrowImage];
        }];
    }
    else
    {
        [self rotateArrowImage];
    }
}

- (void)rotateArrowImage
{
    // 起き上がりフェーズの場合は上向きに反転
    CGFloat rotation = isRising ? M_PI : 0.0f;
    arrowImage.transform = CGAffineTransformMakeRotation(rotation);
}

- (void)validatePitch:(double)pitch
{
    // 角度を度数に変換
    int pitchDegree = round(180 * pitch / M_PI);
    
    if (isRising && pitchDegree > 90 - kPitchDegreeMargin)
    {
        // 起き上がりフェーズかつデバイスの仰角が一定値を超えた場合、起き上がったと判定
        isRising = NO;
        count++;
        [self updateViewAnimated:YES];
    }
    else if (!isRising && pitchDegree < kPitchDegreeMargin)
    {
        // 起き上がりフェーズでないかつデバイスの仰角が一定値を下回った場合、寝たと判定
        isRising = YES;
        [self updateViewAnimated:YES];
    }
}

#pragma mark - Handlers

- (IBAction)butonPressed:(id)sender
{
    // デバイスのハードウェアチェック
    if (!motionManager.deviceMotionAvailable)
    {
        NSLog(@"DeviceMotion is not available.");
        return;
    }
    
    if (isCounting)
    {
        // カウント終了
        [self stopCounting];
    }
    else
    {
        // カウント開始
        [self startCounting];
    }
}

@end
